module Autosolver where

import Minesweeper

data MoveType = Uncover | Flag | None deriving (Eq, Show)

playAutoMove :: Board -> Board
playAutoMove b | fst (nextMove b) == Uncover = uncover b $ snd (nextMove b)
               | fst (nextMove b) == Flag = flag b $ snd (nextMove b)
               | otherwise = b

nextMove :: Board -> (MoveType, Square)
nextMove b | not . null $ uncoverStrat b = (Uncover, head $ uncoverStrat b)
           | not . null $ flagStrat b = (Flag, head $ flagStrat b)
           | not . null $ uncoverStratFallback b = (Uncover, head $ uncoverStratFallback b)
           | otherwise = (None, (0,0))

-- filter:
-- uncovered squares
-- WITH at least one adjacent mine
-- WHERE the number of adjacent mines == number of adjacent flags
-- WHERE number of adjacent covered squares > number of adjacent flags
-- FIND adjacent squares
-- WHICH are covered
-- WHICH are not flagged
uncoverStrat :: Board -> [Square]
uncoverStrat b = 
    filter (not. isFlagged b) $
    filter (isCovered b) $
    concatMap (adjacentSquares b) $
    filter (\s -> adjacentCovereds b s > adjacentFlags b s) $
    filter (\s -> adjacentFlags b s == adjacentMines b s) $
    filter (\s -> adjacentMines b s > 0) $ 
    uncoveredSquares b

-- filter:
-- covered squares
-- WHICH are not flagged
uncoverStratFallback :: Board -> [Square]
uncoverStratFallback b = 
    filter (not. isFlagged b) $
    coveredSquares b

-- filter:
-- uncovered squares
-- WITH at least one adjacent mine
-- WHERE the number of adjacent mines == number of adjacent covered squares
-- FIND adjacent squares
-- WHICH are covered
-- WHICH are not already flagged
flagStrat :: Board -> [Square]
flagStrat b =
    filter (not. isFlagged b) $
    filter (isCovered b) $
    concatMap (adjacentSquares b) $
    filter (\s -> adjacentMines b s == adjacentCovereds b s) $
    filter (\s -> adjacentMines b s > 0) $ 
    uncoveredSquares b